//    lcd-  

//  
// dkorolev@olympiads.ru
//  
// antony@nline.ru

#include "asmegam.h"
#include "lcd.h"

//  

void delay(void)
{
	unsigned char i;
	for(i = 0; i < 100; i++)
		;
}

void delay_100us(void)
{
	register unsigned char i,j;
	for(i = 0; i < 1; i++)
		for(j = 0; j < 200; j++)
			;
}

void delay_5ms(void)
{
	register unsigned char i,j,k;
	for(i = 0; i < 5; i++)
		for(j = 0; j < 40; j++)
			for(k = 0; k < 80; k++)
				;
}

//    ,      

#define LCD_DATA_PORT	PORTA
#define LCD_DATA_PINS	PINA
#define LCD_DATA_DDR	DDRA
#define LCD_EN_DDR		DDRE
#define LCD_RS_DDR		DDRC
#define LCD_RW_DDR		DDRC
#define LCD_EN_PORT		PORTE
#define LCD_RS_PORT		PORTC
#define LCD_RW_PORT		PORTC
#define LCD_E_PIN		BIT(2)
#define LCD_RS_PIN		BIT(6)
#define LCD_RW_PIN		BIT(5)

//     

#define SET_LCD_PORT_OUTPUT	LCD_DATA_DDR = 0xff
#define SET_LCD_PORT_INPUT	LCD_DATA_DDR = 0x00

#define SET_LCD_E			LCD_EN_PORT |= LCD_E_PIN
#define CLEAR_LCD_E			LCD_EN_PORT &= ~LCD_E_PIN

#define SET_LCD_DATA		LCD_RS_PORT |= LCD_RS_PIN
#define SET_LCD_CMD			LCD_RS_PORT &= ~LCD_RS_PIN

#define SET_LCD_READ		LCD_RW_PORT |= LCD_RW_PIN
#define SET_LCD_WRITE		LCD_RW_PORT &= ~LCD_RW_PIN

#define LCD_ON				0b00001100   
#define LCD_CURS_ON			0b00001111
#define LCD_CURS_OFF		0b00001100
#define LCD_CURS_STATIC		0b00001110
#define LCD_OFF				0b00001000
#define LCD_HOME			0b00000010  
#define LCD_CLEAR			0b00000001

#define LCD_INIT_BY_INSTR	0b00110000
#define LCD_MODE_SET        0b00000110
#ifdef LCD_USE_4BIT_INTERFACE
#define LCD_INIT			0b00100000
#define LCD_FUNCTION_SET	0b00101000
#else
#define LCD_INIT			0b00110000
#define LCD_FUNCTION_SET	0b00111000
#endif

//     
byte rus_table[64] = 
{
	//	                                                             
	0x41,0xA0,0x42,0xA1,0xE0,0x45,0xA3,0xA4,0xA5,0xA6,0x4B,0xA7,0x4D,0x48,0x4F,0xA8,
	//	                                                             
	0x50,0x43,0x54,0xA9,0xAA,0x58,0xE1,0xAB,0xAC,0xE2,0xAD,0xAE,0x62,0xAF,0xB0,0xB1,
	//	                                                             
	0x61,0xB2,0xB3,0xB4,0xE3,0x65,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0x6F,0xBE,
	//	                                                             
	0x70,0x63,0xBF,0x79,0xE4,0x78,0xE5,0xC0,0xC1,0xE6,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7
};

//  busy flag
byte lcd_isbusy(void)
{
	byte temp;
	
	SET_LCD_PORT_INPUT;
	CLEAR_LCD_E;
	SET_LCD_READ;
	SET_LCD_CMD;
	SET_LCD_E;
	_NOP();_NOP();_NOP();_NOP();
	_NOP();_NOP();_NOP();_NOP();
	temp = LCD_DATA_PINS;
	CLEAR_LCD_E;
	_NOP();_NOP();_NOP();_NOP();
	_NOP();_NOP();_NOP();_NOP();
	
#ifdef LCD_USE_4BIT_INTERFACE
	SET_LCD_E;
	_NOP();_NOP();_NOP();_NOP();
	CLEAR_LCD_E;
#endif
	
	if(temp & 0b10000000)
		return 1;
	else
		return 0;
}

//   (8- )
void lcd_write(byte data)
{
	SET_LCD_PORT_OUTPUT;
	LCD_DATA_PORT = data;
	SET_LCD_E;
	_NOP();_NOP();_NOP();_NOP();
	CLEAR_LCD_E;
}

//   
void lcd_control(byte cmd)
{
	while(lcd_isbusy())
		;
	CLEAR_LCD_E;
	SET_LCD_WRITE;
	SET_LCD_CMD;
	delay();
    lcd_write(cmd);
}

//        
void putchar0(byte data)
{
	while(lcd_isbusy())
		;
	CLEAR_LCD_E;
	SET_LCD_WRITE;
	SET_LCD_DATA;
	delay();
    lcd_write(data);
}

//         
void outtext0(char* s)
{
	byte i;
	
	while(lcd_isbusy())
		;
	CLEAR_LCD_E;
	SET_LCD_WRITE;
	SET_LCD_DATA;
	delay();
	
	i = 0;
	while(s[i])
	{
		while(lcd_isbusy())
			;
		SET_LCD_PORT_OUTPUT;
		SET_LCD_WRITE;
		SET_LCD_DATA;
		delay();
		lcd_write(s[i]);
		i++;
	}
}

//     
void putchar(byte data)
{
	while(lcd_isbusy())
		;
	CLEAR_LCD_E;
	SET_LCD_WRITE;
	SET_LCD_DATA;
	delay();
	if(data < 0xc0)
		lcd_write(data);
	else
		lcd_write(rus_table[data-0xc0]);
}

//      
void outtext(char* s)
{
	byte i, data;
	
	while(lcd_isbusy())
		;
	CLEAR_LCD_E;
	SET_LCD_WRITE;
	SET_LCD_DATA;
	delay();
	
	i = 0;
	while(s[i])
	{
		while(lcd_isbusy())
			;
		SET_LCD_PORT_OUTPUT;
		SET_LCD_WRITE;
		SET_LCD_DATA;
		data = s[i];
		if(data < 0xc0)
			lcd_write(data);
		else
			lcd_write(rus_table[data-0xc0]);
		i++;
	}
}

//  
void clrscr(void)
{
	lcd_control(LCD_CLEAR);
}

//  
byte rows[] = { 0x80, 0xc0, 0x94, 0xd4 };
void gotoxy(byte x, byte y)
{
	lcd_control(rows[y] + x);
}

// 
void init_lcd(void)
{
	SET_LCD_PORT_OUTPUT;
	LCD_EN_DDR |= LCD_E_PIN;
	LCD_RS_DDR |= LCD_RS_PIN;
	LCD_RW_DDR |= LCD_RW_PIN;
	CLEAR_LCD_E;
	SET_LCD_CMD;
	SET_LCD_WRITE;
	
	delay_5ms();
	delay_5ms();
	delay_5ms();
	delay_5ms();
	
	lcd_write(LCD_INIT_BY_INSTR);
	delay_5ms();
	lcd_write(LCD_INIT_BY_INSTR);
	delay_100us();
	lcd_write(LCD_INIT_BY_INSTR);
	delay();
	lcd_write(LCD_INIT);
	delay();
	lcd_control(LCD_FUNCTION_SET);
	lcd_control(LCD_ON);
	lcd_control(LCD_MODE_SET);
	lcd_control(LCD_CLEAR);
	gotoxy(0,0);
}
