#pragma language=extended

// general

#ifdef AT45D081A
#define AT45_NAME			"AT45D081A"
#define AT45_PAGES_COUNT	4096
#define AT45_PAGE_SIZE		264
#define AT45_BLOCKS_COUNT	512
#else
#ifdef AT45DB321
#define AT45_NAME			"AT45DB321"
#define AT45_PAGES_COUNT	8192
#define AT45_PAGE_SIZE		528
#define AT45_BLOCKS_COUNT	1024
#else
#ifdef AT45DB642
#define AT45_NAME			"AT45DB642"
#define AT45_PAGES_COUNT	8192
#define AT45_PAGE_SIZE		1056
#define AT45_BLOCKS_COUNT	1024
#else
#error "Unknown memory type; set in #define directive befofe including this file."
#endif
#endif
#endif

// commands

#define CMD_BUFFER_1							0x00
#define CMD_BUFFER_2							0x01
#define CMD_BUFFER_1_WRITE						0x84
#define CMD_BUFFER_2_WRITE						0x87
#define CMD_BUFFER_1_READ						0x54
#define CMD_BUFFER_2_READ						0x56
#define CMD_B1_TO_MM_PAGE_PROG_WITH_ERASE		0x83
#define CMD_B2_TO_MM_PAGE_PROG_WITH_ERASE		0x86
#define CMD_B1_TO_MM_PAGE_PROG_WITHOUT_ERASE	0x88
#define CMD_B2_TO_MM_PAGE_PROG_WITHOUT_ERASE	0x89
#define CMD_MM_PAGE_PROG_THROUGH_B1 			0x82
#define CMD_MM_PAGE_PROG_THROUGH_B2 			0x85
#define CMD_AUTO_PAGE_REWRITE_THROUGH_B1		0x58
#define CMD_AUTO_PAGE_REWRITE_THROUGH_B2		0x59
#define CMD_MM_PAGE_TO_B1_COMP					0x60
#define CMD_MM_PAGE_TO_B2_COMP					0x61
#define CMD_MM_PAGE_TO_B1_XFER					0x53
#define CMD_MM_PAGE_TO_B2_XFER					0x55
#define CMD_MAIN_MEMORY_PAGE_READ				0x52
#define CMD_PAGE_ERASE							0x81
#define CMD_BLOCK_ERASE 						0x50
#define CMD_FULL_READ							0xE8

// pins

//hw spi: sck=pb1, mosi=pb2, miso=pb3
//#define PORTSPI		PORTB
//#define PINSPI		PINB
//#define DDRSPI		DDRB
//#define SPI_SCK		BIT(1)
//#define SPI_MOSI	BIT(2)
//#define SPI_MISO	BIT(3)
//at45: cs=pb5, spi=spi
//#define AT45_CS	 	BIT(5)

#define AT45_CS		5 // pb
#define AT45_READY	4 // pb
#define AT45_SCK	1 // pb
#define AT45_MOSI	2 // pb
#define AT45_MISO	3 // pb

// low_level functions

static void delay_45() { ; }

static void init_45(void)
{
	DDRB |= (1<<AT45_SCK);
	DDRB |= (1<<AT45_MOSI);
	DDRB &= ~(1<<AT45_MISO);
	PORTB |= (1<<AT45_MISO);
	
	DDRB |= (1<<AT45_CS);
	PORTB |= (1<<AT45_CS); // no cs at start!
	
	DDRB &= ~(1<<AT45_READY);
	PORTB |= (1<<AT45_READY); // pullup
}

static void init_SPI_45(void)
{
	DDRB |= (1<<AT45_SCK);     
	DDRB |= (1<<AT45_MOSI);
	DDRB &= ~(1<<AT45_MISO);
	PORTB |= (1<<AT45_MISO);
	PORTB |= (1<<AT45_SCK);
    SPCR = 0x5f;
    SPSR = 0;
}

static void close_SPI_45(void)
{
    SPCR = 0;
    SPSR = 0;
	DDRB |= (1<<AT45_SCK);
	PORTB |= (1<<AT45_SCK);
	DDRB &= ~(1<<AT45_MOSI);
	DDRB &= ~(1<<AT45_MISO);
	PORTB |= (1<<AT45_MISO);
	PORTB |= (1<<AT45_MOSI);
}

static unsigned char spi(unsigned char x)
{
	SPDR = x;
	while(!(SPSR & 0x80 /*(1<<SPIF)*/))
		;
	return SPDR;
}

static unsigned char read_SPI_45(void)
{
    return spi(0);
}

static void write_SPI_45(unsigned char data)
{
    spi(data);
}

static void set_CS_45(void)
{
	PORTB |= (1<<AT45_CS);
}

static void clear_CS_45(void)
{
	PORTB &= ~(1<<AT45_CS);
}

static void wait_ready_45(void)
{
	while(!(PINB & (1<<AT45_READY)))
		;
}

// high-level functions
// assume, AT45_init is called first
// address	=	{ 0..AT45_PAGE_SIZE-1 }
// page 	=	{ 0..AT45_PAGES_COUNT-1 }

void AT45_init(void); // first time initialization
void AT45_erase_all(void); // chip erase

unsigned char AT45_get_status(void);

void AT45_read_buffer_begin(unsigned int address);
unsigned char AT45_read(void) { return read_SPI_45(); }
void AT45_read_buffer_end(void);

void AT45_write_buffer_begin(unsigned int address);
void AT45_write(unsigned char data) { write_SPI_45(data); }
void AT45_write_buffer_end(void);

void AT45_mem_to_buf(unsigned int addr);
void AT45_buf_to_mem(unsigned int addr);

void AT45_init(void)
{
	init_45();
	init_SPI_45();
	clear_CS_45();
	write_SPI_45(0x27);
	write_SPI_45(0x4f);
	write_SPI_45(0x37);
	write_SPI_45(0xa0);
	write_SPI_45(0x17);
	write_SPI_45(0x20);
	wait_ready_45();
	set_CS_45();
	close_SPI_45();
}

void AT45_erase_all(void)
{
	unsigned int i;
	wait_ready_45();
	init_SPI_45();
	for(i = 0; i < 64; /*AT45_BLOCKS_COUNT;*/ i++)
	{
		wait_ready_45();
		clear_CS_45();
		write_SPI_45(CMD_BLOCK_ERASE); // ok
		write_SPI_45((char)(i>>4));
		write_SPI_45((char)(i<<4));
		write_SPI_45(0x00);
		set_CS_45();
	}
	close_SPI_45();
	wait_ready_45();
}

unsigned char AT45_get_status(void)
{
	unsigned char data;
	init_SPI_45();
	clear_CS_45();
	write_SPI_45(0x57);
	data = read_SPI_45();
	set_CS_45();
	close_SPI_45();
	return data;
}

void AT45_read_buffer_begin(unsigned int address)
{
	wait_ready_45();
	init_SPI_45();
	clear_CS_45();
	write_SPI_45(CMD_BUFFER_1_READ); // ok
	write_SPI_45(0x00);
	write_SPI_45(0x00);
	write_SPI_45(0x00);
	write_SPI_45(0x00); // 1 don't care
}

void AT45_read_buffer_end(void)
{
	set_CS_45();
	close_SPI_45();
}

void AT45_read_mem_begin(unsigned int addr)
{
	wait_ready_45();
	init_SPI_45();
	clear_CS_45();
	write_SPI_45(CMD_FULL_READ); // ok
	write_SPI_45((char)(addr>>5));
	write_SPI_45((char)(addr<<3));
	write_SPI_45(0x00);
	write_SPI_45(0x00);
	write_SPI_45(0x00);
	write_SPI_45(0x00);
	write_SPI_45(0x00);
}

void AT45_read_mem_end(void)
{
	set_CS_45();
	close_SPI_45();
}

void AT45_write_buffer_begin(unsigned int addr)
{
	wait_ready_45();
	init_SPI_45();
	clear_CS_45();
	write_SPI_45(CMD_BUFFER_1_WRITE); // ok
	write_SPI_45(0x00);
	write_SPI_45((char)(addr>>8));
	write_SPI_45((char)addr);
}

void AT45_write_buffer_end(void)
{
	set_CS_45();
	close_SPI_45();
}

void AT45_buf_to_mem(unsigned int addr)
{
	wait_ready_45();
	init_SPI_45();
	clear_CS_45();
	write_SPI_45(CMD_B1_TO_MM_PAGE_PROG_WITHOUT_ERASE); // ok
	write_SPI_45((char)(addr>>5));
	write_SPI_45((char)(addr<<3));
	write_SPI_45(0x00);
	close_SPI_45();
	set_CS_45();
	wait_ready_45();
}

void AT45_mem_to_buf(unsigned int addr)
{
	wait_ready_45();
	init_SPI_45();
	clear_CS_45();
	write_SPI_45(CMD_MM_PAGE_TO_B1_XFER); // ok
	write_SPI_45((char)(addr>>6));
	write_SPI_45((char)(addr<<2));
	write_SPI_45(0x00);
	close_SPI_45();
	set_CS_45();
	wait_ready_45();
}

